function [W myapprox] = GradMP_MatRec(A, u, s, d1, d2, tol, maxIter)
% Description: Gradient Matching Pursuit (GradMP) algorithm 
% to approximate the vector x from measurements u = A*x.
%
% Usage: [w myapprox] = GradMP_MatRec2(A, u, s, d1, d2, tol, maxIter)
%  Inputs:
%       A = measurement matrix
%       u = (noisy) measurements
%       s = sparsity level
% (d1,d2) = matrix size
%     tol = tolerance used to exit loop
% maxIter = maximum number of iterations
%  Outputs:
%       w = s-sparse approximation of target signal
%myapprox = matrix storing approximation (in the columns) at each iteration
%
% Most recent change - 9/1/2015
%
% Copyright 2015, Nam Nguyen, Deanna Needell, Tina Woolf
%
% This file is part of the Stochastic Greedy Toolbox version 1.0.
%
%    Stochastic Greedy Toolbox is free software: you can redistribute it 
%    and/or modify it under the terms of the GNU General Public License as 
%    published by the Free Software Foundation, either version 3 of the 
%    License, or (at your option) any later version.
%
%    Stochastic Greedy Toolbox is distributed in the hope that it will be 
%    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with Stochastic Greedy Toolbox.  If not, 
%    see <http://www.gnu.org/licenses/>.

%% Set Parameters
M = size(A,1);    % number of rows (measurements)
d = size(A,2);    % number of columns (dimension of signal), d = d1*d2
W = zeros(d1,d2); % initial signal approximation
Gamma_L = [];
Gamma_R = [];
T_L = [];
T_R = [];
k = 0;            % loop index
done = 0;         % while loop flag

%% Main Loop of GradMP
while (~done)
   k = k+1;
   
   % Proxy
   w = W(:);
   r = sum(bsxfun( @times,A,-2/M*(u-A*w) ) ,1);
   R = reshape(r,d1,d2);
   if (~isfinite(sum(sum(R))) )
       % Solution is diverging, so return current solution
       return;
   end

   % Identify
   [U,S,V] = svd(R); % get the best rank-2s approximation of R 
   U = U(:,1:2*s);
   S = S(1:2*s,1:2*s);
   V = V(:,1:2*s);
   Gamma_L = U; Gamma_R = V;
   
   % Merge
   Gamma_hat_L = [Gamma_L, T_L];  % Merging step
   Gamma_hat_R = [Gamma_R, T_R];  % Merging step      
   Gamma = [];
   for i = 1:size(Gamma_hat_L,2)
       Gamma = [Gamma, kron(Gamma_hat_R(:,i), Gamma_hat_L(:,i))];
   end
   
   % Estimate
   % Least squares: solve min ||u - A(b)||_2 where b=Gamma_hat*alpha
   alpha = pinv(A*Gamma)*u;  % Get alpha
   B = Gamma_hat_L*diag(alpha)*Gamma_hat_R';   % Compute B
 
   % Prune
   [U,S,V] = svd(B);   % get the best rank-s approximation of B
   U = U(:,1:s);
   S = S(1:s,1:s);
   V = V(:,1:s);
   T_L = U; T_R = V;
   
   % Update
   W = U*S*V';
   
   % Keep track of approximations at each iteration
   myapprox(1:d,k) = W(:);
   
   % Check exit criteria
   if (norm(u-A*W(:),2) <= tol || k >= maxIter || isnan(norm(u-A*W(:))) || isinf(norm(u-A*W(:))))
       done = 1;
   end    

end

end