function [W myapprox] = IHT_MatRec(A, u, s, gamma, d1, d2, tol, maxIter)
% Description: Iterative Hard Thresholding (IHT) algorithm to approximate
% the vector x from measurements u = A*x.
%
% Usage: [a myapprox] = IHT_MatRec(A, u, s, gamma, d1, d2, tol, maxIter)
%  Inputs:
%         A = measurement matrix
%         u = (noisy) measurements
%         s = rank level
%     gamma = step size
%   (d1,d2) = matrix size
%       tol = tolerance used to exit loop
%   maxIter = maximum number of iterations
%  Outputs:
%         a = s-rank approximation of target signal
%  myapprox = matrix holding column vectors of signal 
%            approximation at each iteration
%
% Most recent change - 9/1/2015
%
% Copyright 2015, Nam Nguyen, Deanna Needell, Tina Woolf
%
% This file is part of the Stochastic Greedy Toolbox version 1.0.
%
%    Stochastic Greedy Toolbox is free software: you can redistribute it 
%    and/or modify it under the terms of the GNU General Public License as 
%    published by the Free Software Foundation, either version 3 of the 
%    License, or (at your option) any later version.
%
%    Stochastic Greedy Toolbox is distributed in the hope that it will be 
%    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with Stochastic Greedy Toolbox.  If not, 
%    see <http://www.gnu.org/licenses/>.

%% Set Parameters
d = size(A,2);    % signal dimension
W = zeros(d1,d2); % initial signal approximation
k = 0;            % loop index
done = 0;         % while loop flag

%% Main Loop of IHT
while (~done)
   k = k+1;
   
   % Proxy
   w = W(:);
   z = w + gamma*A'*(u-A*w);
   Z = reshape(z,d1,d2);
   if (~isfinite(sum(sum(Z))) )
       % Solution is diverging, so return current solution
       return;
   end

   % Identify
   [U,S,V] = svd(Z); % compute SVD of Z 
   U = U(:,1:s);
   S = S(1:s,1:s);
   V = V(:,1:s);
   
   % Estimate
   W = U*S*V'; % best s-rank approximation to Z
   
   % Keep track of approximations at each iteration
   myapprox(1:d,k) = W(:);

   % Check exit criteria
   if (norm(u-A*W(:)) <= tol || k >= maxIter || isnan(norm(u-A*W(:))) ||  isinf(norm(u-A*W(:))))
        done = 1;
   end
   
end

end

