function [Xrec err trueError CPUTime] = StoIHT_MatRec_Approx(y, A, At, X, parameters)
% Description: Stochastic Iterative Hard Thresholding (StoIHT) 
% alglorithm to solve the problem:
% min | b - A(X) |_2    s.t   rank(X) <= r
%
% Usage: [Xrec err trueError CPUTime] = StoIHT_MatRec_Approx(y, A, At, X, parameters)
%  Inputs:
%         y  = (noisy) measurements
%         A  = measurement operator
%         At = Adjoint of A
%         X  = true signal (only used to compute actual error
% parameters = structure containing the following parameters:
%         obsSet    = indices for measurement operator
%         blocksize = block size
%         svdMethod = {'svds','randSVD'} SVD solvers
%         tol       = termination tolerance on absolute error
%         maxIter   = maximum number of iterations
%         VERBOSE   = verbose (1 = on, 0 = off)
%         tol_early = Terminate early when residual change is less than tol_early
%         gamma     = step size
%
%  Outputs:
%       Xrec = recovered signal
%       err  = relative error
%  trueErorr = absolute error
%    CPUTime = run time   
%
% Most recent change - 9/1/2015
%
% Copyright 2015, Nam Nguyen, Deanna Needell, Tina Woolf
%
% This file is part of the Stochastic Greedy Toolbox version 1.0.
%
%    Stochastic Greedy Toolbox is free software: you can redistribute it 
%    and/or modify it under the terms of the GNU General Public License as 
%    published by the Free Software Foundation, either version 3 of the 
%    License, or (at your option) any later version.
%
%    Stochastic Greedy Toolbox is distributed in the hope that it will be 
%    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with Stochastic Greedy Toolbox.  If not, 
%    see <http://www.gnu.org/licenses/>.

if (nargin < 5)
    disp('Not enough arguments');
    Xrec = []; err=  [];
    return;
end

%% Set Parameters

obsSet    = parameters.obsSet;
blocksize = parameters.blocksize;
svdMethod = parameters.svdMethod;
tol       = parameters.tol;
maxIter   = parameters.maxIter;
VERBOSE   = parameters.VERBOSE;
tol_early = parameters.tol_early;
gamma     = parameters.gamma;

cpuinit   = cputime;
temp      = At(y,obsSet);
rows      = size(temp,1); cols = size(temp,2);
N         = rows*cols;
Xhat      = zeros(rows,cols);
m         = length(obsSet);
nb        = floor(m/blocksize);
indices(1:length(obsSet)) = obsSet;
iter       = 1;
err(iter)  = 1;
err2(iter) = 1;

%% Main Loop of StoIHT
while ((err2(iter) > tol)&&(iter <= maxIter))
    
    % Randomize
    ii    = ceil(rand*nb);
    block = blocksize*(ii-1)+1:blocksize*ii;
    
    % Proxy
    R     = -2*At(y(block)-A(Xhat,indices(block)),indices(block));
    B     = Xhat-gamma*R;
   
    % Identify
    switch lower(char(svdMethod(1)))
        case 'svds'
            temp = svdMethod(2); rankX = temp{1};
            [U, S, V] = svds(B, rankX);
        case 'randsvd'
            temp  = svdMethod(2); rankX    = temp{1};
            temp2 = svdMethod(3); overSamp = temp2{1};
            [U, S, V] = randSVD(B,rankX,overSamp);       
        otherwise
            temp = svdMethod(2); rankX = temp{1};
            [U,S,V] = svd(B,0);
            U = U(:,1:min(rankX,size(U,2)));
            V = V(:,1:min(rankX,size(V,2)));
            S = S(1:min(rankX,size(S,1)),1:min(rankX,size(S,2)));
    end
   
    % Estimate
    Xhat = U*S*V';
   
    errTemp = norm(A(Xhat,obsSet)-y)/norm(y);
    trueError(iter) = norm(Xhat-X,'fro');
    if (VERBOSE)
        fprintf('Iter: %d Relative Error: %f, Absolute Error (2-norm): %f, Absolute Error (Frobenius norm): %f\n',iter,errTemp,errTemp*norm(y),trueError(iter));
    end
    
    if ((err-errTemp)/err <= tol_early) %Early termination condition
        err = errTemp;
        if (VERBOSE)
            disp(sprintf('Terminating.'));
        end
        break 
    end

    CPUTime(iter) = cputime-cpuinit;
    iter          = iter + 1;
    err(iter)     = errTemp;
    err2(iter)    = norm(A(Xhat,obsSet)-y);
   
end

% Get final data and return
Xrec = Xhat;

if (VERBOSE)
	disp(sprintf('Final error: %3.6f', errTemp));
end

end
