%% demo_LowRankMatrixRecoveryWithApproximations.m 
% Demonstrates the StoIHT and StoGradMP algorithms in the Low Rank Matrix
% Recovery setting in which the identification, estimation, and pruning
% steps can only be performed approximately. These steps utilize an
% approximate SVD algorithm. The results for the full SVD are included for
% comparison.
%
% Most recent change - 9/1/2015
%
% Copyright 2015, Nam Nguyen, Deanna Needell, Tina Woolf
%
% This file is part of the Stochastic Greedy Toolbox version 1.0.
%
%    Stochastic Greedy Toolbox is free software: you can redistribute it 
%    and/or modify it under the terms of the GNU General Public License as 
%    published by the Free Software Foundation, either version 3 of the 
%    License, or (at your option) any later version.
%
%    Stochastic Greedy Toolbox is distributed in the hope that it will be 
%    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with Stochastic Greedy Toolbox.  If not, 
%    see <http://www.gnu.org/licenses/>.

%% Initialize
clear
close all

addpath(genpath('Operators/'))
addpath(genpath('RandSVD/'))
addpath(genpath('Solvers/'))

%% Declare Algorithm Parameters
N                    = 1024;                                 % The size of the matrix
r                    = 40;                                   % Matrix rank
m_stoiht             = floor(0.3*N^2);                       % The number of measurements to acquire using StoIHT
m_sgradmp            = floor(0.35*N^2);                      % The number of measurements to acquire using StoGradMP
overSamp             = r;                                    % oversampling factor
maxEpochs_stoiht     = 300;                                  % maximum number of epochs using StoIHT
maxEpochs_sgradmp    = 200;                                  % maximum number of epochs using StoGradMP
blocksize_stoiht     = floor(m_stoiht/2);                    % blocksize using StoIHT
blocksize_sgradmp    = floor(m_sgradmp/2);                   % blocksize using StoGradMP
parameters.VERBOSE   = 1;                                    % parameter to print out results to command window
parameters.tol_early = -Inf;                                 % condition to terminate algorithm early
parameters.gamma     = 1/2;                                  % step size for StoIHT
parameters.tol       = 1e-7;                                 % algorithm tolerance
rng('default');

%% Generate rank-r Signal
Xl = randn(N,r); 
Xr = randn(N,r);
W  = Xl*Xr';

%% Generate Sensing Operator/Matrix and Measurements for StoIHT
idx    = randperm(prod(size(W))); 
obsSet = sort(idx(1:m_stoiht),'ascend');		
A      = @(Z,obsSet) Afft(Z,obsSet); 
At     = @(z,obsSet) Atfft(z,obsSet,size(W));	

parameters.obsSet    = obsSet;
parameters.blocksize = floor(m_stoiht/2);  % blocksize
nb_stoiht            = floor(m_stoiht/parameters.blocksize); % number of blocks using StoIHT
parameters.maxIter   = maxEpochs_stoiht*nb_stoiht;% maximum iterations

% Calculate Measurements
y = A(W,obsSet);
    
%% Run StoIHT using full SVD
parameters.svdMethod = {'svds' r 0}; % svdMethod = {'svd method' rank overSamp}
[Wrec_stoiht_svd err_stoiht_svd trueError_stoiht_svd CPUTime_stoiht_svd] = StoIHT_MatRec_Approx(y, A, At, W, parameters);
    
%% Run StoIHT using randSVD
parameters.svdMethod = {'randSVD' r overSamp}; % svdMethod = {'svd method' rank overSamp}
[Wrec_stoiht_randsvd err_stoiht_randsvd trueError_stoiht_randsvd CPUTime_stoiht_randsvd] = StoIHT_MatRec_Approx(y, A, At, W, parameters); 

%% Generate Sensing Operator/Matrix and Measurements for StoGradMP
idx    = randperm(prod(size(W))); 
obsSet = sort(idx(1:m_sgradmp),'ascend');		
A      = @(Z,obsSet) Afft(Z,obsSet); 
At     = @(z,obsSet) Atfft(z,obsSet,size(W));	

parameters.obsSet    = obsSet;
parameters.blocksize = floor(m_sgradmp/2);  % blocksize
nb_sgradmp           = floor(m_sgradmp/parameters.blocksize);% number of blocks using StoGradMP
parameters.maxIter   = maxEpochs_sgradmp*nb_sgradmp;% maximum iterations

% Calculate Measurements
y = A(W,obsSet);

%% Run StoGradMP using full SVD
parameters.svdMethod = {'svds' r 0}; % svdMethod = {'svd method' rank overSamp}
[Wrec_sgradmp_svd err_sgradmp_svd trueError_sgradmp_svd CPUTime_sgradmp_svd] = StoGradMP_MatRec_Approx(y, A, At, W, parameters);

%% Run StoGradMP using randSVD
parameters.svdMethod = {'randSVD' r overSamp}; % svdMethod = {'svd method' rank overSamp}
[Wrec_sgradmp_randsvd err_sgradmp_randsvd trueError_sgradmp_randsvd CPUTime_sgradmp_randsvd] = StoGradMP_MatRec_Approx(y, A, At, W, parameters);

%% Display Results

% Error is measurmed as norm(X-Xhat,'fro')
disp(['StoIHT with full SVD: Error ' num2str(trueError_stoiht_svd(end)) ', CPU Time ' num2str(CPUTime_stoiht_svd(end))]); 
disp(['StoIHT with randSVD:  Error ' num2str(trueError_stoiht_randsvd(end)) ', CPU Time ' num2str(CPUTime_stoiht_randsvd(end))]);
disp(['StoGradMP with full SVD: Error ' num2str(trueError_sgradmp_svd(end)) ', CPU Time ' num2str(CPUTime_sgradmp_svd(end))]);
disp(['StoGradMP with randSVD:  Error ' num2str(trueError_sgradmp_randsvd(end)) ', CPU Time ' num2str(CPUTime_sgradmp_randsvd(end))]);

% Example Results:
% StoIHT with full SVD: Error 3.2848e-07, CPU Time 1705.99
% StoIHT with randSVD:  Error 2.3519e-07, CPU Time 723.55
% StoGradMP with full SVD: Error 0.0076881, CPU Time 10359.96
% StoGradMP with randSVD:  Error 0.0059808, CPU Time 2861.61