function [a myapprox] = IHT(A, u, s, tol, maxIter)
% Description: Iterative Hard Thresholding (IHT) algorithm to approximate
% the vector x from measurements u = A*x.
%
% Usage: [a myapprox] = IHT(A, u, s, tol, maxIter)
%  Inputs:
%         A = measurement matrix
%         u = (noisy) measurements
%         s = sparsity level
%       tol = tolerance used to exit loop
%   maxIter = maximum number of iterations
%  Outputs:
%         a = s-sparse approximation of target signal
%  myapprox = matrix holding column vectors of signal 
%            approximation at each iteration
%
% Most recent change - 9/1/2015
%
% Copyright 2015, Nam Nguyen, Deanna Needell, Tina Woolf
%
% This file is part of the Stochastic Greedy Toolbox version 1.0.
%
%    Stochastic Greedy Toolbox is free software: you can redistribute it 
%    and/or modify it under the terms of the GNU General Public License as 
%    published by the Free Software Foundation, either version 3 of the 
%    License, or (at your option) any later version.
%
%    Stochastic Greedy Toolbox is distributed in the hope that it will be 
%    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with Stochastic Greedy Toolbox.  If not, 
%    see <http://www.gnu.org/licenses/>.

%% Set Parameters
d    = size(A,2);  % signal dimension
a    = zeros(d,1); % initial signal approximation
k    = 0;          % loop index
done = 0;          % while loop flag

%% Main Loop of IHT
while (~done)
   k = k+1;
   
   % Proxy 
   z = a + A'*(u-A*a);
   
   % Estimate
   [z_sort ix_z] = sort(abs(z),'descend');
   a = zeros(d,1);
   a(ix_z(1:s)) = z(ix_z(1:s));
   
   % Keep track of approximations at each iteration
   myapprox(1:d,k) = a;

   % Check exit criteria
   if (norm(u-A*a,2) <= tol || k >= maxIter)
        done = 1;
   end
   
end

end

