function [w myapprox] = StoGradMP(A, u, s, blocksize, p, tol, maxIter)
% Description: Stochastic Gradient Matching Pursuit (SGradMP) algorithm 
% to approximate the vector x from measurements u = A*x.
%
% Usage: [w myapprox] = StoGradMP(A, u, s, p, tol, maxIter)
%  Inputs:
%         A = measurement matrix
%         u = (noisy) measurements
%         s = sparsity level
% blocksize = block size
%         p = vector of probabilities of length size(A,1)
%       tol = tolerance used to exit loop
%   maxIter = maximum number of iterations
%  Outputs:
%         w = s-sparse approximation of target signal
%  myapprox = matrix storing approximation (in the columns) at each iteration
%
% Most recent change - 9/1/2015
%
% Copyright 2015, Nam Nguyen, Deanna Needell, Tina Woolf
%
% This file is part of the Stochastic Greedy Toolbox version 1.0.
%
%    Stochastic Greedy Toolbox is free software: you can redistribute it 
%    and/or modify it under the terms of the GNU General Public License as 
%    published by the Free Software Foundation, either version 3 of the 
%    License, or (at your option) any later version.
%
%    Stochastic Greedy Toolbox is distributed in the hope that it will be 
%    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with Stochastic Greedy Toolbox.  If not, 
%    see <http://www.gnu.org/licenses/>.

%% Set Parameters
d = size(A,2);           % number of columns (dimension of signal)
m = size(A,1);           % number of rows (measurements)
w = zeros(d,1);          % initial signal approximation
Gamma = [];
T = [];
k = 0;                   % loop index
done = 0;                % while loop flag
nb = floor(m/blocksize); % number of blocks

%% Main Loop of StoGradMP
while (~done)
   k = k+1;
   
   % Randomize
   ii = ceil(rand*nb); 
   block = blocksize*(ii-1)+1:blocksize*ii;   
   
   % Proxy
   r = -2*A(block,:)'*(u(block)-A(block,:)*w);
   
   % Identify
   [r_sort ix_r] = sort(abs(r),'descend');
   Gamma = ix_r(1:2*s); % identify large components
   
   % Merge
   Gamma_hat = union(Gamma,T);
   
   % Estimate
   b(Gamma_hat) = pinv(A(:,Gamma_hat))*u;
   
   % Prune
   [b_sort ix_b] = sort(abs(b),'descend');
   T = ix_b(1:s);
   
   % Update
   w = zeros(d,1);
   w(T) = b(T);
   
   % Keep track of approximations at each iteration
   myapprox(1:d,k) = w;
   
   % Check exit criteria
   if (norm(u-A*w,2) <= tol || k >= maxIter)
       done = 1;
   end   

end


end