%% demo_SparseVectorRecovery.m 
% Demonstrates the StoIHT and StoGradMP algorithms in the Sparse Vector
% Recovery setting, as well as the IHT and GradMP algorithms for
% comparison.
%
% Most recent change - 9/1/2015
%
% Copyright 2015, Nam Nguyen, Deanna Needell, Tina Woolf
%
% This file is part of the Stochastic Greedy Toolbox version 1.0.
%
%    Stochastic Greedy Toolbox is free software: you can redistribute it 
%    and/or modify it under the terms of the GNU General Public License as 
%    published by the Free Software Foundation, either version 3 of the 
%    License, or (at your option) any later version.
%
%    Stochastic Greedy Toolbox is distributed in the hope that it will be 
%    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with Stochastic Greedy Toolbox.  If not, 
%    see <http://www.gnu.org/licenses/>.

%% Initialize
clear all; 
close all;

%% Declare Algorithm Parameters
n         = 20;                % number of trials
d         = 256;               % signal dimension
s         = [8 12 20];         % sparsity level of signal
m         = 5:10:250;          % number of measurements
tol_alg   = 1e-7;              % tolerance to quit in algorithm
tol_rec   = 1e-6;              % tolerance to declare perfect recovery
maxEpochs = 500;               % maximum epochs in algorithms
gamma     = 1;                 % step size for iterative hard thresholding algorithms (IHT and StoIHT)

% Flag for which algorithm to run (1 = yes, 0 = no)
iht.flag     = 1; 
stoiht.flag  = 1;
gradmp.flag  = 1;
sgradmp.flag = 1;

% Correct recovery data for each algorithm
iht.CorrRec     = zeros(length(s),length(m));
stoiht.CorrRec  = zeros(length(s),length(m));
gradmp.CorrRec  = zeros(length(s),length(m));
sgradmp.CorrRec = zeros(length(s),length(m));

%% Run Experiments
for is = 1:length(s) % for each sparsity level
    for im = 1:length(m) % for each measurement value
        b  = min(m(im),s(is));% block size
        nb = floor(m(im)/b); % number of blocks
        p  = ones(1,nb)*1/nb; % set probabilities
        for in = 1:n % for each trial
            
            % Generate s(is)-sparse signal
            x                  = zeros(d,1);
            rand_perm          = randperm(d);
            support_indices    = rand_perm(1:s(is));
            x(support_indices) = randn(length(support_indices),1); % Gaussian signal
            
            % Generate measurement matrix
            A = randn(m(im),d)/sqrt(m(im)); % Gaussian measurement matrix
            
            % Compute measurements
            y = A*x; % measurements
            
            % Run Greedy Algorithms
            
            % IHT
            if (iht.flag)
                [iht.xhat] = IHT(A, y, s(is), tol_alg, maxEpochs);

                if (norm(x-iht.xhat,2) <= tol_rec)
                    iht.CorrRec(is,im) = iht.CorrRec(is,im) + 1; 
                end               
            end
            
            % StoIHT
            if (stoiht.flag)                
                [stoiht.xhat] = StoIHT(A, y, s(is), b, gamma, p, tol_alg, maxEpochs*nb);

                if (norm(x-stoiht.xhat,2) <= tol_rec)
                    stoiht.CorrRec(is,im) = stoiht.CorrRec(is,im) + 1; 
                end                 
            end          

            % GradMP
            if (gradmp.flag)
                [gradmp.xhat] = GradMP(A, y, s(is), tol_alg, maxEpochs); 
                
                if (norm(x-gradmp.xhat,2) <= tol_rec)
                    gradmp.CorrRec(is,im) = gradmp.CorrRec(is,im) + 1; 
                end                      
            end
            
            % StoGradMP
            if (sgradmp.flag)
                [sgradmp.xhat] = StoGradMP(A, y, s(is), b, p, tol_alg, maxEpochs*nb);
                
                if (norm(x-sgradmp.xhat,2) <= tol_rec)
                    sgradmp.CorrRec(is,im) = sgradmp.CorrRec(is,im) + 1; 
                end   
            end
            
        end
        
        % Command window output to display progress
        disp(['Completed sparsity ' num2str(s(is)) ' with ' num2str(m(im)) ' measurements']);
        
    end
end

%% Plot Percentage of Signals Recovered

markers = {'+','o','*','.','x','s','d','^','v','>','<','p','h'};

if (iht.flag)
    figure;
    hold all;
    % plot curve for each sparsity level
    s_toPlot = s;
    for ii = 1:length(s_toPlot)
        s_index = find(s == s_toPlot(ii));
        plot(m, iht.CorrRec(s_index,:)/n*100,'-','Marker',markers{ii},'linewidth',2);
        LegendString{ii} = sprintf('s = %d', s(s_index));
    end
    xlabel('Number of Measurements m','fontsize',14);
    ylabel('Percentage of Signals Recovered (%)','fontsize',14);
    title('Percentage of Signals Recovered using IHT','fontsize',16);
    set(gca,'fontsize',12);
    legend(LegendString,4); 
    ylim([0 101]);
    grid on;   
end
            
if (stoiht.flag)
    figure;
    hold all;
    % plot curve for each sparsity level
    s_toPlot = s;
    for ii = 1:length(s_toPlot)
        s_index = find(s == s_toPlot(ii));
        plot(m, stoiht.CorrRec(s_index,:)/n*100,'-','Marker',markers{ii},'linewidth',2);
        LegendString{ii} = sprintf('s = %d', s(s_index));
    end
    xlabel('Number of Measurements m','fontsize',14);
    ylabel('Percentage of Signals Recovered (%)','fontsize',14);
    title('Percentage of Signals Recovered using StoIHT','fontsize',16);
    set(gca,'fontsize',12);
    legend(LegendString,4);
    ylim([0 101]);
    grid on;
end    

if (gradmp.flag)
    figure;
    hold all;
    % plot curve for each sparsity level
    s_toPlot = s;
    for ii = 1:length(s_toPlot)
        s_index = find(s == s_toPlot(ii));
        plot(m, gradmp.CorrRec(s_index,:)/n*100,'-','Marker',markers{ii},'linewidth',2);
        LegendString{ii} = sprintf('s = %d', s(s_index));
    end
    xlabel('Number of Measurements m','fontsize',14);
    ylabel('Percentage of Signals Recovered (%)','fontsize',14);
    title('Percentage of Signals Recovered using GradMP','fontsize',16);
    set(gca,'fontsize',12);
    legend(LegendString,4);
    ylim([0 101]);
    grid on;
end

if (sgradmp.flag)
    figure;
    hold all;
    % plot curve for each sparsity level
    s_toPlot = s;
    for ii = 1:length(s_toPlot)
        s_index = find(s == s_toPlot(ii));
        plot(m, sgradmp.CorrRec(s_index,:)/n*100,'-','Marker',markers{ii},'linewidth',2);
        LegendString{ii} = sprintf('s = %d', s(s_index));
    end
    xlabel('Number of Measurements m','fontsize',14);
    ylabel('Percentage of Signals Recovered (%)','fontsize',14);
    title('Percentage of Signals Recovered using StoGradMP','fontsize',16);
    set(gca,'fontsize',12);
    legend(LegendString,4);
    ylim([0 101]);
    grid on;
end

